'use strict';

const font = require('ansi-colors');

const _ = require('../util_modules');
const store = require('../configstore');
const Credential = require('../internal/credential');
const Authorize = require('./login');
const Revoke = require('./revoke');
const prompt = require('../prompt');
const i18next = require('i18next');

module.exports = {
	login: (options) => {
		let user = store.get('user');
		let token = store.get('credential');
		
		if (user && token && !options.parent.force) {
			_.LOGGER.message(i18next.t('login_already') + font.cyan.bold(user.Email));
			return Promise.resolve(user);
		}
		//intially when the user login or logout default workspace  will be deleted
		store.delete('default_workspace');
		return prompt
			.ask(
				prompt.question(
					'collectUsage',
					i18next.t('login_collect_logs'),
					{
						type: 'confirm',
						defaultAns: true
					}
				)
			)
			.then((ans) => {
				store.set('usage', ans.collectUsage);
				let localhost = _.OPTION.getCommandOption(options)('localhost', true);
				return new Authorize(localhost).init().then((result) => {
					store.set('user', result.user);
					store.set('scopes', result.scopes);
					store.set('oAuthCreatedTime', Date.now());
					Credential.init(result.token).persistMinimal('credential');
					_.LOGGER.success(i18next.t('login_success') + font.cyan.bold(result.user.Email));
					process.exit(1);
				});
			});
	},
	logout: () => {
		let tokenObj = store.get('credential', null);
		if (_.JS.isNull(tokenObj)) {
			return _.LOGGER.error(
				i18next.t('login_error')
			);
		}
		let user = store.get('user', null);
		let next;
		if (!_.JS.isNull(user)) {
			next = prompt.ask(
				prompt.question(
					'consent',
					i18next.t('logout_warning'),
					{ type: 'confirm', defaultAns: true }
				)
			);
		} else {
			next = Promise.resolve({ consent: true });
		}
		return next.then((ans) => {
			if (!ans.consent) {
				_.LOGGER.message(i18next.t('logout_error'));
				return Promise.resolve();
			}
			let credential = Credential.init(tokenObj, false);
			return new Revoke(credential, 'credential').init().then(() => {
				_.LOGGER.success(i18next.t('logout_success'));
			});
		});
	},

	autoLogout: () => {
		return new Promise(function (resolve, reject) {
		let tokenObj = store.get('credential', null);
		if (!_.JS.isNull(tokenObj)) {
			let credential = Credential.init(tokenObj, false);
			return new Revoke(credential, 'credential').init().then(() => {
				_.LOGGER.success(i18next.t('auto_logout'));
				process.exit(1);
				resolve();
			});
		}
	});
	} 
};
