/*
Copyright (c) 2017, ZOHO CORPORATION
License: MIT
*/

var chalk = require('chalk');
var { existsSync } = require('fs-extra');
var fs = require('fs');
var fileTreeParser = require('../utils/file-tree-parser.js');
var i18next = require('i18next');
var service = require('./../utils/service.js');

module.exports = {
  run : function(projectRootDir, rulesList) {

    var errorStack = {};
    
    if(!existsSync(projectRootDir+'/app')){
      console.log(chalk.red(i18next.t("validate_appmissing")));
      process.exit(1);
    }
   
    var appFiles = fs.readdirSync(projectRootDir+'/app');
    
    if(appFiles.length == 0){
      console.log(chalk.red(i18next.t("validate_appempty")));
      process.exit(1);
    }
     
    
    var serviceName = service.getServiceName(projectRootDir);
    var fileTreeObject = fileTreeParser.parse(projectRootDir, serviceName);

    for( var i=0; i<rulesList.length; i++) {
      var ruleModule = require('../rules/'+ rulesList[i]);
      var errors = ruleModule.run(projectRootDir, fileTreeObject);
      if(Array.isArray(errors) && errors.length > 0) {
        errorStack[ruleModule.name] = errors;
      }
    }
    if( Object.keys(errorStack).length > 0 ) {

      for( var rule in errorStack ) {
        console.log(chalk.bold.red('\n'+rule+ ' :- \n'));
        for( var i=0; i<errorStack[rule].length; i++ ) {
          var errorString = errorStack[rule][i];
          console.log(chalk.red(' ==> '+ errorString));
        }
      }
      process.exit(1);
    } else {
      return true;
    }
  }
};
