'use strict';

const winston = require('winston');
const path = require('path');
const ansiStrip = require('strip-ansi');
const CONSTANTS = require('./util_modules/lib/constants');

module.exports = winston.createLogger({
	transports: [
		new winston.transports.File({
			name: 'FileLog',
			filename: path.join(process.cwd(), CONSTANTS.FILENAME.log),
			level: 'debug',
			handleExceptions: true,
			format: winston.format.combine(
				winston.format.printf((info) => {
					return `[${info.level}] [${new Date().toISOString()}] : ${ansiStrip(
						info.message
					)}`;
				})
			)
		}),
		new winston.transports.Console({
			name: 'ConsoleLog',
			level:
				process.env.DEBUG || process.argv.slice(2).includes('--verbose') ? 'debug' : 'info',
			handleExceptions: false,
			format: winston.format.combine(
				winston.format.colorize(),
				winston.format.printf((info) => {
					return `${info.message}`;
				})
			)
		})
	],
	exitOnError: false
});
