/*
Copyright (c) 2017, ZOHO CORPORATION
License: MIT
*/

var fs = require('fs');
var path = require('path');
var i18next = require('i18next');
var CONST = require('../commands/constants.js');
var util = require('../utils');
var isString = util.isString;
var isUndefined = util.isUndefined;
var isObject = util.isObject;

var SERVICE_NAME = CONST.SERVICE.CATALYST; // constant
var MANIFEST_KEY = ["service", "client", "file_location"];
var ALLOWED_ACCOUNTS_URL = ["https://accounts.localzoho.com", "https://accounts.zoho.com", "https://accounts.csez.zohocorpin.com", "https://accounts.zoho.com.cn", "https://accounts.zoho.eu", "https://accounts.zoho.in"];

function validatePluginManifest(projectRootDir) {
  var errors = [];
  var pluginManifest = require(path.join(projectRootDir, CONST.PLUGIN_MANIFEST));

  let manifestKeys = Object.keys(pluginManifest);
 
  //plugin manifest mandatory key validation
  for (var i = 0; i < MANIFEST_KEY.length; i++) {
    if (manifestKeys.indexOf(MANIFEST_KEY[i]) === -1) {
      errors.push("\"" + MANIFEST_KEY[i] + "\"" + i18next.t('pluginmanifest_keymissing'));
    }
  }

  var filelocation = pluginManifest.file_location, client = pluginManifest.client;

  if (!isUndefined(client)) {
    if (!isObject(client)) {
      errors.push(i18next.t('catalyst_pluginmanifest_client_invalid'));
    } else {
      var clientId = client.id, scope = client.scope, accountUrl = client["accounts-url"];
      if (isUndefined(clientId)) {
        errors.push(i18next.t('catalyst_pluginmanifest_missingclientid'));
      } else if (!isString(clientId) || clientId.trim() === "") {
        errors.push(i18next.t('catalyst_pluginmanifest_invalidclientid'));
      }
      if (isUndefined(scope)) {
        errors.push(i18next.t('catalyst_pluginmanifest_client_missingscope'));
      } else if (!isString(scope) || scope.trim() === "") {
        errors.push(i18next.t('catalyst_pluginmanifest_client_invalidscope'));
      }
      if (isUndefined(accountUrl)) {
        errors.push(i18next.t('catalyst_pluginmanifest_missingaccounturl'));
      } else if (ALLOWED_ACCOUNTS_URL.indexOf(accountUrl) < 0) {
        errors.push(i18next.t('catalyst_pluginmanifest_invalidaccounturl'));
      }
    }
  }

  if (!isUndefined(filelocation)) {
    if (!isString(filelocation) || filelocation.trim() === "") {
      errors.push(i18next.t('catalyst_pluginmanifest_invalidfilelocationtype'));
    } else {
      var startsWithSlash = filelocation.indexOf('/') === 0;
      if (startsWithSlash) {
        if (!fs.existsSync(path.join(projectRootDir, filelocation))) {
          errors.push(i18next.t('catalyst_pluginmanifest_missingfileinappsource'));
        }
      } else {
        errors.push(i18next.t('catalyst_pluginmanifest_invalidfilelocation') + name);
      }
    }
  }

  return errors;
}
module.exports = {

  name: i18next.t('pluginmanifestrule_name'),
  run: function (projectRootDir, fileTree) {
    return validatePluginManifest(projectRootDir);
  }
}