/*
Copyright (c) 2017, ZOHO CORPORATION
License: MIT
*/

var fse = require('fs-extra');
var path = require('path');
var CONST = require('../commands/constants');
var i18next = require('i18next');
var service = require('./../utils/service.js');
var FILE_NAME_REGEX = /^[A-Za-z0-9_.$-]{0,}$/
module.exports = {

  name: i18next.t('filecheck_name'),
  run: function (projectRootDir, fileTree) {

    var errorStack = [];

    var totalNumberOfFiles = 0;
    var projectRoot = fileTree.projectRoot;
    for (var key in fileTree) {
      if (['projectRoot', 'appBase', 'misc'].indexOf(key) !== -1) continue;
      var assets = fileTree[key];

      totalNumberOfFiles += assets.length;

      for (var j = 0; j < assets.length; j++) {
        var filePath = assets[j];
        var fileName = path.basename(filePath);
        if(fileName.lastIndexOf('.')===0){
          errorStack.push(i18next.t("empty_file_name").replace("{filename}",fileName));
        }

        var serviceName = service.getServiceName(projectRootDir);
        var fileStat = fse.statSync(path.join(projectRoot, filePath));
        if (fileStat.size > CONST.FILE_SIZE_LIMIT) {
          errorStack.push(i18next.t('filecheck_prj_hme') + filePath + i18next.t('filecheck_fileexceed') + (CONST.FILE_SIZE_LIMIT/1000000) + i18next.t('filecheck_filemb'));
        }

        if(!FILE_NAME_REGEX.test(fileName)){
          errorStack.push(i18next.t("file_name_regex_fail").replace("{filename}",fileName));
        }

        maxFileNameLen = serviceName==CONST.SERVICE.CRM ? CONST.MAX_FILE_NAME_LEN_CRM : CONST.MAX_FILE_NAME_LEN;

        if(fileName.length>maxFileNameLen){
          errorStack.push(i18next.t('file_name_max_len').replace("{filename}",fileName).replace("{max_length}",maxFileNameLen));
        }
        
      } 

    }

    totalNumberOfFiles += Array.isArray(fileTree['translations']) ? fileTree['translations'].length : 0;

    if( totalNumberOfFiles > CONST.FILES_COUNT_LIMIT ) {
      errorStack.push(i18next.t('filecheck_rule_maxfile')+ CONST.FILES_COUNT_LIMIT);
    }

    return errorStack;
  }
}
