'use strict';

module.exports = {
	isMac: () => {
		return process && process.platform === 'darwin';
	},
	isWindows: () => {
		return (
			process && (process.platform === 'win32' || /^(msys|cygwin)$/.test(process.env.OSTYPE))
		);
	},
	/**
	 * Override a value with supplied environment variable if present.
	 */
	envOverride: (envname, value, coerce) => {
		if (process.env[envname] && process.env[envname].length) {
			if (coerce) {
				try {
					return coerce(process.env[envname], value);
				} catch (e) {
					return value;
				}
			}
			return process.env[envname];
		}
		return value;
	},
	get: (variable, fallback = null) => {
		let value = process.env[variable];
		return value === undefined ? fallback : value;
	}
};
