/*
Copyright (c) 2017, ZOHO CORPORATION
License: MIT
*/
var CONST = require('../commands/constants');
var service = require('../utils/service.js');
var fs = require('fs');
var path = require('path');
var i18next = require('i18next');
const _ = require('../util_modules');
var chalk = require('chalk');
var { existsSync, ensureFileSync, createWriteStream } = require('fs-extra');

var ALLOWED_EXTNS = CONST.ALLOWED_EXTNS;

function getAssetArrayFromExt(assetObj, extnName, serviceName) {

  if (ALLOWED_EXTNS[serviceName].images.indexOf(extnName) >= 0) { return assetObj.images; }
  else if (ALLOWED_EXTNS[serviceName].scripts.indexOf(extnName) >= 0) { return assetObj.scripts; }
  else if (ALLOWED_EXTNS[serviceName].styles.indexOf(extnName) >= 0) { return assetObj.styles; }
  else if (ALLOWED_EXTNS[serviceName].views.indexOf(extnName) >= 0) { return assetObj.views; }
  else if (ALLOWED_EXTNS[serviceName].multimedia.indexOf(extnName) >= 0) { return assetObj.multimedia; }
  else if (ALLOWED_EXTNS[serviceName].translations.indexOf(extnName) >= 0) { return assetObj.translations; }
  else if (ALLOWED_EXTNS[serviceName].data.indexOf(extnName) >= 0) { return assetObj.data; }
  else return assetObj.misc;

}

function _parserUtil(currDir, fileTreeObject, serviceName) {
  if (!currDir) { return; }

  var currPath = path.join(fileTreeObject.projectRoot, currDir);
  var currFiles = fs.readdirSync(currPath);

  for (var i = 0, len = currFiles.length; i < len; i++) {

    var file = currFiles[i];
    var absPath = path.join(fileTreeObject.projectRoot, currDir, file);

    var fileStat = fs.lstatSync(absPath);
    if (fileStat.isDirectory()) {
      //need to check the folder name regex
      if(!/^[A-Za-z0-9_$-]{0,}$/i.test(file)){
        console.log(chalk.red("Folder Name should contain only a-zA-Z0-9_-$. Invalid Folder Name - "+file));
        process.exit(1);
      }
      var newPath = currDir + path.sep + file;
      _parserUtil(newPath, fileTreeObject, serviceName);
    } else {

      var extName = path.extname(file);
      if(file.lastIndexOf('.')===0){
        extName=file;
      }
      var filePath = currDir + path.sep + file;

      var translationJSONPrefix = 'app' + path.sep + 'translations'; // .json file in translations folder check
      if (ALLOWED_EXTNS[serviceName].translations.indexOf(extName) >= 0 && currDir.indexOf(translationJSONPrefix) === 0) {
        fileTreeObject.translations.push(filePath);
      } else if(extName == ".dre") {
        //Dre functions were allowed only if its in server source folder and has version number as folder name
        if(currDir.includes("app/server/source")){
          var dreFolderNameArray = currDir.split("/");
          var dreFolderName = dreFolderNameArray[dreFolderNameArray.length-1].trim();
          if(!isNaN(dreFolderName) || dreFolderName.toLowerCase() == "latest"){
            getAssetArrayFromExt(fileTreeObject, extName, serviceName).push(filePath);
          }
        }
      } else {
        getAssetArrayFromExt(fileTreeObject, extName, serviceName).push(filePath);
      }
    }

  }

  return fileTreeObject;
}

module.exports = {
  parse: function (path, serviceName) {

    var currAppBase = 'app';
    if(!existsSync(path+'/app')){
      console.log(chalk.red(i18next.t("validate_appmissing")));
      process.exit(1);
    }
    var appFiles = fs.readdirSync(path+'/app');
    if(appFiles.length == 0){
      console.log(chalk.red(i18next.t("validate_appempty")));
      process.exit(1);
    }
    var fileTreeObject = {
      'projectRoot': path,
      'appBase': currAppBase,
      'images': [],
      'scripts': [],
      'styles': [],
      'views': [],
      'misc': [],
      'translations': [],
      'multimedia': [],
      'data': []
    };

    var serviceNames = service.getServiceNames().map(function (obj) { return obj.value; });
    if (serviceName === undefined || serviceNames.indexOf(serviceName) === -1) {
      serviceName = "ZOHO";
    }
    // Initiating the parser
    var fileParsetree = _parserUtil(currAppBase, fileTreeObject, serviceName);
    if(fileParsetree.misc && fileParsetree.misc.length > 0){
      _.LOGGER.error(chalk.red(i18next.t("pack_error")));
      for (var i = 0 ; i < fileParsetree.misc.length; i++) {
        _.LOGGER.info(chalk.red(' ==> '+fileParsetree.misc[i]));
      }
      _.LOGGER.info(chalk.bold.red(i18next.t('package_note')));
     // process.exit(1);
    }
    return fileParsetree;
  }
};
